--#########################################################
-- PALTextFontRAM
--#########################################################

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

Library UNISIM;
use UNISIM.vcomponents.all;

--#########################################################
-- entity
--#########################################################

entity PALTextFontRAM is
	port
	(
		font_clk : in  std_logic;
		font_en  : in  std_logic;
		font_addr: in  std_logic_vector(13 downto 0);
		font_do  : out std_logic
	);
end;

--#########################################################
-- architecture
--#########################################################

architecture arch of PALTextFontRAM is

	-----------------------------------------------
	-- font BRAM
	-----------------------------------------------

	signal addr_tmp: std_logic_vector(14 downto 0);
	signal data_tmp: std_logic_vector(31 downto 0);


begin

	-----------------------------------------------
	-- font BRAM
	-----------------------------------------------

	RAMB16_inst:RAMB16
	generic map
	(

		WRITE_MODE_A       => "WRITE_FIRST", -- WRITE_FIRST, READ_FIRST or NO_CHANGE
		READ_WIDTH_A       => 1,             -- Valid values are 1,2,4,9,18 or 36
		WRITE_WIDTH_A      => 1,             -- Valid values are 1,2,4,9,18 or 36
		DOA_REG            => 1,             -- Optional output registers on the A port (0 or 1)
		INIT_A             => X"000000000",  -- Initial values on A output port
		SRVAL_A            => X"000000000",  -- Port A output value upon SSR assertion
		RAM_EXTENSION_A    => "NONE",        -- "UPPER", "LOWER" or "NONE" when cascaded
		INVERT_CLK_DOA_REG => FALSE,         -- Invert clock on A port output registers (TRUE or FALSE)

		WRITE_MODE_B       => "WRITE_FIRST", -- WRITE_FIRST, READ_FIRST or NO_CHANGE
		READ_WIDTH_B       => 1,             -- Valid values are 1,2,4,9,18 or 36
		WRITE_WIDTH_B      => 1,             -- Valid values are 1,2,4,9,18 or 36
		DOB_REG            => 1,             -- Optional output registers on the B port (0 or 1)
		INIT_B             => X"000000000",  -- Initial values on B output port
		SRVAL_B            => X"000000000",  -- Port B output value upon SSR assertion
		RAM_EXTENSION_B    => "NONE",        -- "UPPER", "LOWER" or "NONE" when cascaded
		INVERT_CLK_DOB_REG => FALSE,         -- Invert clock on B port output registers (TRUE or FALSE)


		SIM_COLLISION_CHECK => "ALL",        -- Collision check enable "ALL", "WARNING_ONLY",
		                                     -- "GENERATE_X_ONLY" or "NONE"


		-- The following INIT_xx declarations specify the initial contents of the RAM
		--                          3               2               1               0
		--            . . . . . . . | . . . . . . . | . . . . . . . | . . . . . . . |
		INIT_00 => X"00081C3E3E3E1400001C3E223E2A3E1C001C222A2236221C0000000000000000",
		INIT_01 => X"0000000C0C000000001C083E3E1C080000083E3E081C1C0800081C3E3E1C0800",
		INIT_02 => X"000C12122C3038003F3F212D2D213F3F00001E12121E00003F3F3F33333F3F3F",
		INIT_03 => X"00002A1C361C2A00000636342C342C3000060E0C0828180800081C081C22221C",
		INIT_04 => X"001400141414141400081C3E083E1C080010181C1E1C181000040C1C3C1C0C04",
		INIT_05 => X"1C081C3E083E1C08001E1E0000000000001C2218140C221C002828282C2A2A3C",
		INIT_06 => X"0000080C3E0C0800000008183E18080000081C3E0808080800080808083E1C08",
		INIT_07 => X"000008081C1C3E3E00003E3E1C1C0808000014143E141400003E020202000000",
		INIT_08 => X"00143E14143E1400000000000012363600080008081C1C080000000000000000",
		INIT_09 => X"0000000000040C0C002C122A040A0A04003232040810262600080E100C021C04",
		INIT_0A => X"000008083E0808000000141C3E1C140000040808080808040008040404040408",
		INIT_0B => X"0000020408102000000C0C0000000000000000003E000000040C0C0000000000",
		INIT_0C => X"001C22201C20221C003E02041820221C001C080808080C08001C22262A32221C",
		INIT_0D => X"000404040810203E001C22221E020418001C22201E02023E0010103E12141810",
		INIT_0E => X"040C0C000C0C0000000C0C000C0C0000000C10203C22221C001C22221C22221C",
		INIT_0F => X"000800081820221C000408102010080400003E00003E00000010080402040810",
		INIT_10 => X"001C22020202221C001E22221E22221E0022223E2222221C001C023A2A3A221C",
		INIT_11 => X"003C22223A02221C000202021E02023E003E02021E02023E001E22222222221E",
		INIT_12 => X"0022120A060A1222001C222220202020001C08080808081C002222223E222222",
		INIT_13 => X"001C22222222221C00222222322A262200222222222A3622003E020202020202",
		INIT_14 => X"001C22201C02221C002222121E22221E002C122A2222221C000202021E22221E",
		INIT_15 => X"00142A2A2A2A22220008142222222222001C222222222222000808080808083E",
		INIT_16 => X"001C04040404041C001E02020408101E00080808142222220022221408142222",
		INIT_17 => X"3F000000000000000000000000221408001C10101010101C0000201008040200",
		INIT_18 => X"001C2202221C0000001E2222221E0202003C223C201C00000000000000080C0C",
		INIT_19 => X"1C203C22223C0000000404041E040418001C021E221C0000003C2222223C2020",
		INIT_1A => X"00120A060A1202020C12101010180010001808080808000800121212120E0202",
		INIT_1B => X"001C2222221C000000121212120E00000022222A2A1600000018080808080808",
		INIT_1C => X"001C201C021C0000000E0404241A0000203C2222223C0000021E2222221E0000",
		INIT_1D => X"00143E2A22220000000814222222000000141A121212000000081404041E0400",
		INIT_1E => X"0018040406040418001E020C101E000006081C12121200000012120C12120000",
		INIT_1F => X"00003E2222361C080000000000000A14000C10103010100C0008080800080808",
		INIT_20 => X"003C223C201C001C001C021E221C003000141A12121200120C081C220202221C",
		INIT_21 => X"0C081C2202221C00003C223C201C141C003C223C201C000C003C223C201C0014",
		INIT_22 => X"0018080808080014001C021E221C000C001C021E221C0014001C021E221C001C",
		INIT_23 => X"00223E22361C141C00223E22140800140018080808080004001808080808001C",
		INIT_24 => X"000C1212120C001C003A0A0A3E0A0A3C003C0A3E281E0000003E021E023E0030",
		INIT_25 => X"00141A121212000600141A121212001C000C1212120C0006000C1212120C0014",
		INIT_26 => X"010E161A1C200000000C121212120014000C121212120C1206081C1212120014",
		INIT_27 => X"040A08081C0828100000221408142200001E262A2A2A323C003A24041E042418",
		INIT_28 => X"00141A1212120018000C1212120C00180018080808080018003C223C201C0018",
		INIT_29 => X"001E000C1212120C003C003C223C201C00121A1612000A14001212120E000A14",
		INIT_2A => X"003810221C0A120200000020203F0000001E21352D35291E001C22020C080008",
		INIT_2B => X"0000001224120000000000241224000000081C1C080800080020382A340A1202",
		INIT_2C => X"08080808080808083F2A3F153F2A3F15152A152A152A152A0015002A0015002A",
		INIT_2D => X"00223E221408000C00223E221408001C00223E2214080018080808080F080808",
		INIT_2E => X"0A0A0A0A0B080F000A0A0A0A0A0A0A0A0A0A0A0A0B080B0A001E212D252D211E",
		INIT_2F => X"080808080F00000000083E083E08142200081C02021C0800000000000F080B0A",
		INIT_30 => X"0808080838080808080808083F000000000000003F0808080000000038080808",
		INIT_31 => X"00223E2214081428003C223C201C1428080808083F080808000000003F000000",
		INIT_32 => X"0A0A0A0A3B003F00000000003F003B0A0A0A0A0A3A023E00000000003E023A0A",
		INIT_33 => X"00221C2222221C220A0A0A0A3B003B0A000000003F003F000A0A0A0A3A023A0A",
		INIT_34 => X"003E021E023E0014003E021E023E001C001C24242E24241C000C121C0804020C",
		INIT_35 => X"001C0808081C001C001C0808081C00180000000000080808003E021E023E000C",
		INIT_36 => X"3F3F3F3F3F3F3F3F0808080838000000000000000F080808001C0808081C0014",
		INIT_37 => X"000000003F3F3F3F001C0808081C000C00080808000808083F3F3F3F00000000",
		INIT_38 => X"000C121212120C06000C121212120C1C020E12120E120E00000C121212120C18",
		INIT_39 => X"06020E120E02060002020E1212120000000C1212120C0A14000C12120C000A14",
		INIT_3A => X"000C121212120006000C12121212001C000C12121212001806020E12120E0206",
		INIT_3B => X"0000000000040C0C0000000000001C00000808081422001806081C1212120018",
		INIT_3C => X"0020382A340B160300003E00003E000000001C00081C0800000000001C000000",
		INIT_3D => X"000000181C000000000008003E000800001C2218140C221C002828282C2A2A3C",
		INIT_3E => X"000000000404060400000000040000000000000014000000000000000C12120C",
		INIT_3F => X"000000000000000000001E1E1E1E0000000000000E04080600000000060C040E"
	)
	port map
	(
		CLKA   => font_clk,        --  1-bit A port clock input
		ENA    => font_en,         --  1-bit A port enable input
		WEA    => (others => '0'), --  2-bit A port write enable input
		ADDRA  => addr_tmp,        -- 14-bit A port address input
		DIA    => (others => '0'), -- 16-bit A port data input
		DIPA   => (others => '0'), --  2-bit A port parity data input
		DOA    => data_tmp,        -- 16-bit A port data output
		DOPA   => open,            --  2-bit A port parity data output
		REGCEA => '1',             --  1-bit A port register enable input
		SSRA   => '0',             --  1-bit A port set/reset input


		CLKB   => '0',             --  1 bit B port clock input
		ENB    => '0',             --  1-bit B port enable input
		WEB    => (others => '0'), --  2-bit B port write enable input
		ADDRB  => (others => '0'), -- 14-bit B port address input
		DIB    => (others => '0'), -- 16-bit B port data input
		DIPB   => (others => '0'), --  2-bit B port parity data input
		DOB    => open,            -- 16-bit B port data output
		DOPB   => open,            --  2-bit B port parity data output
		REGCEB => '1',             --  1-bit B port register enable input
		SSRB   => '0',             --  1-bit B port set/reset input

		CASCADEOUTA => open,  -- 1-bit cascade A output
		CASCADEINA  => '0',   -- 1-bit cascade A input
		CASCADEOUTB => open,  -- 1-bit cascade B output
		CASCADEINB  => '0'    -- 1-bit cascade B input
	);


	addr_tmp <= '0' & font_addr;

	font_do <= data_tmp(0);

end;

--#########################################################
-- end
--#########################################################
