--#########################################################
-- palgen_tb
--#########################################################

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.NUMERIC_STD.all;

use STD.TEXTIO.all;

use WORK.TBBenchPkg.all;

--#########################################################
-- entity
--#########################################################

entity palgen_tb is
end;

--#########################################################
-- architecture
--#########################################################

architecture arch of palgen_tb is

	-----------------------------------------------
	-- test signals
	-----------------------------------------------

	-- filename and dir
	constant tbfiledir : string := "Sources/PAL/TestBenches";
	constant tbfilename: string := "palgen_tb.txt";


	-- clock and info
	constant tbspec: TestBenchSpec :=
	(
		10 ns,			-- period
		 5 ns,			-- duty
		 2 ns,			-- hold
		20 ns,			-- offset
		TB_CLK_SDR	-- mode (TB_CLK_SDR,TB_CLK_DDR)

--		31.250 ns,			-- period
--		15.625 ns,			-- duty
--		 5.000 ns,			-- hold
--		50.000 ns,			-- offset
--		TB_CLK_SDR	-- mode (TB_CLK_SDR,TB_CLK_DDR)
	);


	-- testbench clock and index
	signal tbclk: std_logic;
	signal tbidx: std_logic_vector(7 downto 0);
	signal tbmod: std_logic_vector(3 downto 0);

	-----------------------------------------------
	-- test component
	-----------------------------------------------

	component PALGen is
	generic
	(
		CLK_MHZ: integer := 100 -- multiple of 12.5 MHz
	);
	port
	(
		clk  : in  std_logic;
		pixel: out std_logic;
		porch: out std_logic;
		color: out std_logic;
		hpos : out std_logic_vector(9 downto 0);
		vpos : out std_logic_vector(9 downto 0)
	);
	end component;

	constant CLK_MHZ: integer := 100; -- multiple of 12.5 MHz

	signal clk  : std_logic;
	signal pixel: std_logic;
	signal porch: std_logic;
	signal color: std_logic;
	signal hpos : std_logic_vector(9 downto 0);
	signal vpos : std_logic_vector(9 downto 0);

	-----------------------------------------------
	-- stimulus signals
	-----------------------------------------------

	procedure getStimulus(variable bench: inout TestBench) is
	begin
--		benchGet(bench,uio_buttons);
	end;

	procedure nextStimulus(variable bench: inout TestBench) is
		variable idle: boolean := false;
		variable stop: boolean := false;
	begin
	--	idle := false;
		idle := (tbmod = X"1");
	--			   (tbmod = X"1" and locked = '0')
	--			or (tbmod = X"2" and locked = '1'); -- ??
		stop := false;

		if stop then
			wait;
		elsif idle then
			benchIdle(bench,tbclk,tbmod,tbidx);
		else
			benchRead(bench,tbclk,tbmod,tbidx);
			getStimulus(bench);
			benchStep(bench,tbclk,tbmod,tbidx);
		end if;
	end;

begin

	-----------------------------------------------
	-- test component(s)
	-----------------------------------------------

	test_inst:PALGen
	generic map
	(
		CLK_MHZ => CLK_MHZ
	)
	port map
	(
		clk   => clk,
		pixel => pixel,
		porch => porch,
		color => color,
		hpos  => hpos,
		vpos  => vpos
	);

	clk <= tbclk;

	-----------------------------------------------
	-- simulation process
	-----------------------------------------------

	process
		variable outtxt: line;
		variable bench : TestBench;
	begin

		benchCreate(tbfiledir&'\'&tbfilename,tbspec,bench);

		benchStart(bench,tbclk,tbmod,tbidx);

		while true loop
			nextStimulus(bench);
		end loop;

		benchDelete(bench);

		wait;
	end process;

end;

--#########################################################
-- end
--#########################################################
